@@ -207,8 +207,6 @@ Y.extend(DRAGDROP, Y.Base, {
         if (!this.in_group(drag)) {
             return;
         }
-        // Store the nodes current style, so we can restore it later.
-        this.originalstyle = drag.get('node').getAttribute('style');
         // Set some general styles here
         drag.get('node').setStyle('opacity', '.25');
         drag.get('dragNode').setStyles({
@@ -227,7 +225,10 @@ Y.extend(DRAGDROP, Y.Base, {
             return;
         }
         //Put our general styles back
-        drag.get('node').setAttribute('style', this.originalstyle);
+        drag.get('node').setStyles({
+            visibility: '',
+            opacity: '1'
+        });
         this.drag_end(e);
     },
 
@@ -337,7 +338,7 @@ Y.extend(DRAGDROP, Y.Base, {
      */
     find_element_text: function(n) {
         // The valid node types to get text from.
-        var nodes = n.all('h2, h3, h4, h5, span:not(.actions):not(.menu-action-text), p, div.no-overflow, div.dimmed_text');
+        var nodes = n.all('h2, h3, h4, h5, span, p, div.no-overflow, div.dimmed_text');
         var text = '';
 
         nodes.each(function () {
@@ -374,8 +375,9 @@ Y.extend(DRAGDROP, Y.Base, {
         // Build the list of drop targets.
         var droplist = Y.Node.create('<ul></ul>');
         droplist.addClass('dragdrop-keyboard-drag');
-        var listitem;
-        var listitemtext;
+        var listitem,
+            listlink,
+            listitemtext;
 
         // Search for possible drop targets.
         var droptargets = Y.all('.' + this.samenodeclass + ', .' + this.parentnodeclass);
