@@ -25,8 +25,13 @@
  */
 
 require('../config.php');
-require_once('signup_form.php');
+require_once($CFG->dirroot . '/user/editlib.php');
 
+// Try to prevent searching for sites that allow sign-up.
+if (!isset($CFG->additionalhtmlhead)) {
+    $CFG->additionalhtmlhead = '';
+}
+$CFG->additionalhtmlhead .= '<meta name="robots" content="noindex" />';
 
 if (empty($CFG->registerauth)) {
     print_error('notlocalisederrormessage', 'error', '', 'Sorry, you may not use this page.');
@@ -41,9 +46,25 @@ if (!$authplugin->can_signup()) {
 $PAGE->https_required();
 
 $PAGE->set_url('/login/signup.php');
-$PAGE->set_context(get_context_instance(CONTEXT_SYSTEM));
+$PAGE->set_context(context_system::instance());
 
-$mform_signup = new login_signup_form();
+// Override wanted URL, we do not want to end up here again if user clicks "Login".
+$SESSION->wantsurl = $CFG->wwwroot . '/';
+
+if (isloggedin() and !isguestuser()) {
+    // Prevent signing up when already logged in.
+    echo $OUTPUT->header();
+    echo $OUTPUT->box_start();
+    $logout = new single_button(new moodle_url($CFG->httpswwwroot . '/login/logout.php',
+        array('sesskey' => sesskey(), 'loginpage' => 1)), get_string('logout'), 'post');
+    $continue = new single_button(new moodle_url('/'), get_string('cancel'), 'get');
+    echo $OUTPUT->confirm(get_string('cannotsignup', 'error', fullname($USER)), $logout, $continue);
+    echo $OUTPUT->box_end();
+    echo $OUTPUT->footer();
+    exit;
+}
+
+$mform_signup = $authplugin->signup_form();
 
 if ($mform_signup->is_cancelled()) {
     redirect(get_login_url());
@@ -51,11 +72,16 @@ if ($mform_signup->is_cancelled()) {
 } else if ($user = $mform_signup->get_data()) {
     $user->confirmed   = 0;
     $user->lang        = current_language();
-    $user->firstaccess = time();
+    $user->firstaccess = 0;
     $user->timecreated = time();
     $user->mnethostid  = $CFG->mnet_localhost_id;
     $user->secret      = random_string(15);
     $user->auth        = $CFG->registerauth;
+    // Initialize alternate name fields to empty strings.
+    $namefields = array_diff(get_all_user_name_fields(), useredit_get_required_name_fields());
+    foreach ($namefields as $namefield) {
+        $user->$namefield = '';
+    }
 
     $authplugin->user_signup($user, true); // prints notice and link to login/index.php
     exit; //never reached
